/*
 * I pinched this from Portfolio2022; more specifically, here:
 *   https://git.matt-barnes.co.uk/mb2g17/Portfolio2022/src/branch/master/utils/filter-description-generator.ts
 */

/**
 * Converts an array of words into an English-readable list.
 *
 * USAGE:
 * []                             => default output;
 * ["Go"]                         => "Go";
 * ["Go", "React"]                => "Go or React";
 * ["Go", "React", "Unity"]       => "Go, React, or Unity";
 * ["Go", "React", "Unity", "Qt"] => "Go, React, Unity, or Qt".
 *
 * @param words - The array of words to list into English
 * @param defaultOutput - The default string to return if the list is empty
 * @returns An English-readable list of the given words, separated by commas and 'or'.
 */
const wordsToEnglishList = (words: string[], defaultOutput: string): string => {
    if (words.length === 0) return defaultOutput;
    if (words.length === 1) return words[0];
    if (words.length === 2) return `${words[0]} or ${words[1]}`;
    if (words.length === 3) return `${words[0]}, ${words[1]}, or ${words[2]}`;

    const tail = wordsToEnglishList(words.slice(1), defaultOutput);
    return `${words[0]}, ${tail}`;
};

export const generateFilterDescription = (
    type: string,
    genres: string[],
    languages: string[],
    frameworks: string[],
    technologies: string[]
) => {
    // Project type
    let displayedType = type;
    if (type !== 'Applications' && genres.length > 0) {
        displayedType = displayedType.toLowerCase();
    }

    // Genre
    let genreList = wordsToEnglishList(genres, '');
    if (type === 'Applications') {
        genreList = '';
    }

    // Language
    const languageList = wordsToEnglishList(languages, 'any language');

    // Framework
    const frameworkList = wordsToEnglishList(frameworks, 'any');
    const frameworkThe = frameworks.length > 0 ? 'the ' : '';
    const frameworkPlural = frameworks.length > 1 ? 's' : '';

    // Technology
    const technologyList = wordsToEnglishList(technologies, 'any technology');

    // Aggregating
    let rv = '';
    rv += `${genreList} ${displayedType} written in ${languageList}, `;
    rv += `using ${frameworkThe}${frameworkList} framework${frameworkPlural}`;

    if (technologies.length > 0) {
        rv += `, created with ${technologyList}.`;
    } else {
        rv += '.';
    }

    // Trimming
    rv = rv.trim();

    return rv;
};
