import type { Project } from '$lib/types';
import { markdownToProject } from '$lib/utils';

export const prerender = false;

export const load = (async ({ url }) => {
    // Get all projects
    const mdFiles = import.meta.glob("./*.md");
    let projects: Project[] = [];

    for (const mdPath in mdFiles) {
        const rawProj: any = await mdFiles[mdPath]();
        const slug = mdPath.slice(2, -3);
        const proj = markdownToProject(rawProj.metadata, rawProj.default, slug);

        projects = [...projects, proj];
    }

    // Sort projects by date
    projects.sort((p1, p2) => p2.dateOfCompletion.getTime() - p1.dateOfCompletion.getTime());

    // Get languages, frameworks, etc.
    const languages: Set<string> = new Set();
    const frameworks: Set<string> = new Set();
    const technologies: Set<string> = new Set();
    const genres: Set<string> = new Set();
    for (const proj of projects) {
        proj.languages.forEach(l => languages.add(l));
        proj.frameworks.forEach(f => frameworks.add(f));
        proj.technologies.forEach(t => technologies.add(t));
        proj.genres.forEach(g => genres.add(g));
    }

    return {
        projects: projects,
        queryParams: url.searchParams.toString(),
        languages: [...languages],
        frameworks: [...frameworks],
        technologies: [...technologies],
        genres: [...genres],
    };
});
